"""
ddadevops provide tools to support builds combining gopass, 
terraform, dda-pallet, aws & hetzner-cloud.

"""

from .credential import gopass_credential_from_env_path, gopass_credential_from_path
from .devops_build import DevopsBuild, create_devops_build_config
from .devops_terraform_build import DevopsTerraformBuild, create_devops_terraform_build_config
from .hetzner_mixin import HetznerMixin, add_hetzner_mixin_config
from .aws_mixin import AwsMixin, add_aws_mixin_config
from .dda_pallet_mixin import DdaPalletMixin, add_dda_pallet_mixin_config

__version__ = "0.4.0.dev12"


from os import path
from json import load
from subprocess import run
from python_terraform import *
from .python_util import filter_none
from .devops_build import DevopsBuild, create_devops_build_config


def create_devops_terraform_build_config(stage, project_root_path, build_commons_path, module,
                                         account_name, additional_vars, 
                                         build_dir_name='target',
                                         terraform_build_commons_dir_name='terraform', 
                                         output_json_name='output.json',
                                         use_workspace=True):
    ret = create_devops_build_config(
        stage, project_root_path, build_commons_path, module, build_dir_name)
    ret.update({'account_name': account_name,
                'additional_vars': additional_vars,
                'terraform_build_commons_dir_name': terraform_build_commons_dir_name,
                'output_json_name': output_json_name,
                'use_workspace': use_workspace})
    return ret


class DevopsTerraformBuild(DevopsBuild):

    def __init__(self, project, config):
        super().__init__(project, config)
        self.additional_vars = config['additional_vars']
        self.terraform_build_commons_dir_name = config['terraform_build_commons_dir_name']
        self.output_json_name = config['output_json_name']
        self.use_workspace = config['use_workspace']

    def terraform_build_commons_path(self):
        mylist = [self.build_commons_path,
                  self.terraform_build_commons_dir_name]
        return '/'.join(filter_none(mylist))

    def project_vars(self):
        ret = {'stage': self.stage}
        if self.module:
            ret['module'] = self.module
        if self.additional_vars:
            ret.update(self.additional_vars)
        return ret

    def initialize_build_dir(self):
        super().initialize_build_dir()
        run('cp -f ' + self.terraform_build_commons_path() +
            '* ' + self.build_path(), shell=True)
        run('cp *.tf ' + self.build_path(), shell=True)
        run('cp *.properties ' + self.build_path(), shell=True)
        run('cp *.tfars ' + self.build_path(), shell=True)

    def init_client(self):
        tf = Terraform(working_dir=self.build_path())
        tf.init()
        if self.use_workspace:
            try:
                tf.workspace('select', slef.stage)
            except:
                tf.workspace('new', self.stage)
        return tf

    def write_output(self, tf):
        result = tf.output(json=IsFlagged)
        with open(self.build_path() + self.output_json_name, "w") as output_file:
            output_file.write(json.dumps(result))

    def read_output_json(self):
        with open(self.build_path() + self.output_json_name, 'r') as f:
            return load(f)

    def plan(self):
        tf = self.init_client()
        tf.plan(capture_output=False, var=self.project_vars())

    def apply(self, p_auto_approve=False):
        tf = self.init_client()
        tf.apply(capture_output=False, auto_approve=p_auto_approve,
                 var=self.project_vars())
        self.write_output(tf)

    def destroy(self, p_auto_approve=False):
        tf = self.init_client()
        tf.destroy(capture_output=False, auto_approve=p_auto_approve,
                   var=self.project_vars())

    def tf_import(self, tf_import_name, tf_import_resource,):
        tf = self.init_client()
        tf.import_cmd(tf_import_name, tf_import_resource,
                      capture_output=False, var=self.project_vars())



from python_terraform import *
from .credential import gopass_credential_from_env_path
from .devops_terraform_build import DevopsTerraformBuild


def add_aws_mixin_config(config, account_name):
    config.update({'AwsMixin':
                   {'account_name': account_name}})
    return config


class AwsMixin(DevopsTerraformBuild):

    def __init__(self, project, config):
        super().__init__(project, config)
        aws_mixin_config = config['AwsMixin']
        self.account_name = aws_mixin_config['account_name']

    def backend_config(self):
        return "backend." + self.account_name + "." + self.stage + ".properties"

    def project_vars(self):
        ret = super().project_vars()
        ret.update({'account_name': self.account_name})
        return ret

    def init_client(self):
        tf = Terraform(working_dir=self.build_path())
        tf.init(backend_config=self.backend_config())
        try:
            tf.workspace('select', slef.stage)
        except:
            tf.workspace('new', self.stage)
        return tf

    def plan(self):
        tf = self.init_client()
        tf.plan(capture_output=False, var=self.project_vars(),
                var_file=self.backend_config())



from subprocess import run
from .python_util import filter_none


def create_devops_build_config(stage, project_root_path, build_commons_path, module, build_dir_name='target'):
    return {'stage': stage,
            'project_root_path': project_root_path,
            'build_commons_path': build_commons_path,
            'module': module,
            'build_dir_name': build_dir_name}

class DevopsBuild:

    def __init__(self, project, config):
        self.stage = config['stage']
        self.project_root_path = config['project_root_path']
        self.build_commons_path = config['build_commons_path']
        self.module = config['module']
        self.build_dir_name = config['build_dir_name']
        self.project = project
        project.set_property("devops_build", self)

    def name(self):
        return self.project.name

    def build_path(self):
        mylist = [self.project_root_path,
                  self.build_dir_name,
                  self.name(),
                  self.module]
        return '/'.join(filter_none(mylist))

    def initialize_build_dir(self):
        run('rm -rf ' + self.build_path(), shell=True)
        run('mkdir -p ' + self.build_path(), shell=True)



from subprocess import check_output, call
import os
import sys

def gopass_credential_from_env_path (env_var):
    env_path = os.environ.get(env_var, None)
    return gopass_credential_from_path(env_path)

def gopass_credential_from_path (path):
    credential = None

    if path:
        print('get credential for: ' + path)
        if sys.version_info.major == 3:
            credential = check_output(['gopass', path], encoding='UTF-8')
        else:
            credential = check_output(['gopass', path])

    return credential




from subprocess import check_output
import sys

def execute(cmd):
    if sys.version_info.major == 3:
        output = check_output(cmd, encoding='UTF-8')
    else:
        output = check_output(cmd)
    return output

def filter_none(list):
    return [x for x in list if x is not None]



#!/usr/bin/env python

from setuptools import setup
from setuptools.command.install import install as _install

class install(_install):
    def pre_install_script(self):
        pass

    def post_install_script(self):
        pass

    def run(self):
        self.pre_install_script()

        _install.run(self)

        self.post_install_script()

if __name__ == '__main__':
    setup(
        name = 'ddadevops',
        version = '0.4.0.dev12',
        description = 'tools to support builds combining gopass, terraform, dda-pallet, aws & hetzner-cloud',
        long_description = '',
        author = 'meissa GmbH',
        author_email = 'buero@meissa-gmbh.de',
        license = 'Apache Software License',
        url = 'https://github.com/DomainDrivenArchitecture/dda-devops-build',
        scripts = [],
        packages = ['ddadevops'],
        namespace_packages = [],
        py_modules = [],
        classifiers = [
            'License :: OSI Approved :: Apache Software License',
            'Programming Language :: Python',
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.6',
            'Programming Language :: Python :: 3.7',
            'Topic :: Software Development :: Build Tools'
        ],
        entry_points = {},
        data_files = [],
        package_data = {},
        install_requires = [],
        dependency_links = [],
        zip_safe = True,
        cmdclass = {'install': install},
        keywords = '',
        python_requires = '',
        obsoletes = [],
    )



from .credential import gopass_credential_from_env_path
from .devops_terraform_build import DevopsTerraformBuild


def add_hetzner_mixin_config(config):
    config.update({'HetznerMixin':
                   {'HETZNER_API_KEY_PATH_ENVIRONMENT': 'HETZNER_API_KEY_PATH'}})
    return config


class HetznerMixin(DevopsTerraformBuild):

    def __init__(self, project, config):
        super().__init__(project, config)
        hetzner_mixin_config = config['HetznerMixin']
        self.hetzner_api_key = gopass_credential_from_env_path(
            hetzner_mixin_config['HETZNER_API_KEY_PATH_ENVIRONMENT'])

    def project_vars(self):
        ret = super().project_vars()
        if self.hetzner_api_key:
            ret['hetzner_api_key'] = self.hetzner_api_key
        return ret
